#include<iostream>
#include<fstream>
using namespace std;

struct Pixel
{
	unsigned char blue;
	unsigned char green;
	unsigned char red;
};

struct BitmapFileHeader
{
	char identifier[3];
	unsigned int fileSize;
	unsigned int reserved;
	unsigned int pixelStartingAddress;
	void display();
	void ReadFromFile(ifstream& fin);
	void WriteToFile(ofstream& fout);
};

void BitmapFileHeader::ReadFromFile(ifstream& fin)
{
	fin.read(identifier,2);
	identifier[2] = '\0';
	fin.read((char*)&fileSize,4);
	fin.read((char*)&reserved,4);
	fin.read((char*)&pixelStartingAddress,4);
}

void BitmapFileHeader::WriteToFile(ofstream& fout)
{
	fout.write(identifier,2);
	fout.write((char*)&fileSize,4);
	fout.write((char*)&reserved,4);
	fout.write((char*)&pixelStartingAddress,4);
}


struct BitmapImageHeader
{
	unsigned int sizeOfHeader;
	unsigned int width;
	unsigned int  height;
	unsigned short colorPlanes;
	unsigned short bitsPerPixel;
	unsigned int compressionMethod;
	unsigned int imageSize;
	unsigned int  horizontalResolution;
	unsigned int  verticalResolution;
	unsigned int  colorsInPalette;
	unsigned int  importantColors;

	void ReadFromFile(ifstream& fin);
	void WriteToFile(ofstream& fout);
};

void BitmapImageHeader::ReadFromFile(ifstream& fin)
{
	fin.read((char*)&sizeOfHeader,4);
	fin.read((char*)&width,4);
	fin.read((char*)&height ,4);
	fin.read((char*)&colorPlanes,2);
	fin.read((char*)&bitsPerPixel,2);
	fin.read((char*)&compressionMethod,4);
	fin.read((char*)&imageSize,4);
	fin.read((char*)& horizontalResolution,4);
	fin.read((char*)&verticalResolution ,4);
	fin.read((char*)&colorsInPalette ,4);
	fin.read((char*)&importantColors ,4);
}

void BitmapImageHeader::WriteToFile(ofstream& fout)
{
	fout.write((char*)&sizeOfHeader,4);
	fout.write((char*)&width,4);
	fout.write((char*)&height ,4);
	fout.write((char*)&colorPlanes,2);
	fout.write((char*)&bitsPerPixel,2);
	fout.write((char*)&compressionMethod,4);
	fout.write((char*)&imageSize,4);
	fout.write((char*)& horizontalResolution,4);
	fout.write((char*)&verticalResolution ,4);
	fout.write((char*)&colorsInPalette ,4);
	fout.write((char*)&importantColors ,4);
}

void BitmapFileHeader::display()
{
	cout << identifier << endl;
	cout << fileSize << endl;
	cout << pixelStartingAddress << endl;
}

struct Image
{
	BitmapFileHeader fileHeader;
	BitmapImageHeader imageHeader;
	//Pixel pixels[200000]; //static memory allocation
	//create a pointer for dynamic memory allocation later
	Pixel* pixels;

	//array of pixels
	Image(char fileName[]);
	~Image();
	void  invertColors();
	void save(char fileName[]);
};

Image::~Image()
{
	delete[] pixels;  //dynamically free memory
}

Image::Image(char fileName[])
{
	ifstream fin(fileName, ios_base::binary );
	fileHeader.ReadFromFile(fin);
	imageHeader.ReadFromFile(fin);
	pixels = new Pixel[imageHeader.width * imageHeader.height]; //dynamic memory allocation
	//read the pixels
	for(int i = 0; i < imageHeader.width * imageHeader.height;i++)
	{
		fin.read((char*)&(pixels[i].blue),1);
		fin.read((char*)&(pixels[i].green),1);
		fin.read((char*)&(pixels[i].red),1);
	}
}

void  Image::invertColors()
{
	for(int i = 0; i < imageHeader.width * imageHeader.height;i++)
	{
		pixels[i].blue = 255 - pixels[i].blue;
		pixels[i].green = 255 - pixels[i].green;
		pixels[i].red = 255 - pixels[i].red;
	}
}
void Image::save(char fileName[])
{
	ofstream fout(fileName, ios_base::binary );
	fileHeader.WriteToFile(fout);
	imageHeader.WriteToFile(fout);
	//write pixels
	for(int i = 0; i < imageHeader.width * imageHeader.height;i++)
	{
		fout.write((char*)&(pixels[i].blue),1);
		fout.write((char*)&(pixels[i].green),1);
		fout.write((char*)&(pixels[i].red),1);
	}
}

void main()
{
	//int* x;
	//while(true)
	//{
	//	x = new int[1000];
	//	delete[] x;
	//}

	Image image("Penguins.bmp");
	image.invertColors();
	image.save("out.bmp");

	//ofstream fout("stuff.dat", ios_base::binary );
	////pointer - a variable that holds an address
	//int i = 7777444;
	////& - address of ... the varible to the right
	////when declaring a variable * means it is a pointer
	//int* pi =  &i;
	//char *pc = (char*)pi;  //type cast to lie
	//fout.write( pc,4);
	//fout.write( (char*)&i,4);
	//fout.close();

	//int j;
	//ifstream fin("stuff.dat",ios_base::binary);
	//fin.read((char*)&j,4);
	//cout << j << endl;
	//fin.close();
}